#define DEBUG_LEVEL     0

#define DRIVER_NAME "nVidia Riva Integrated Clock driver"
#define DRIVER_REV  "0.1"

#include <kgi/module.h>
#include <nVidia/nvriva.h>
#include <nVidia/nv4ref.h>
#include <nVidia/nvreg.h>

#include "pll.h"

static const struct pll_limits dclk_pll =
{
	13500000,                       /* Fref                         */
	{ 1, 1 },                       /* Factor                       */
	{ 100000000, 250000000 },       /* FVCO range                   */
	{ 0, 4 },                       /* Post-VCO divider range       */
	{ 1, 255 },                     /* Dividend range               */
	{ 1, 255 }                       /* Divisor range                */
};

static struct kgi_clock clock =
{
	"nVidia",			/* Manufacturer         */
	"Riva",				/* Model                */
	GM_ALL,                         /* Supported modes      */
	0,                              /* Special capabilities */
	{ 0x7FFF, 0x7FFF },             /* Maximum resolution   */
	NULL,				/* Private data hook	*/
	CLK_TYPE_PROG,                  /* Clock type           */
	{{
		0, 250000000,		/* Clock range          */
	}}
};

/* An SDAC command word is as follows:
 * **      [0:7]   s->div
 * **      [8:15]  s->mul
 * **      [16:18] s->p
 * */

#define INVALID_COMMAND ((uint) -1)

static kgi_uint pll_command(struct pll_state *s, struct kgi_mode *mode)
{
	return  s->f;
}

static void pll_set(struct kgi_display *dpy, kgi_uint cmd)
{
	GET_PRIVATE;	

	priv->clock = cmd;
}

#define NO_KGIM_CLOCK_INIT

kgi_sint kgim_clock_init(struct kgi_card *card, struct kgi_display *dpy)
{
	NOTICE(DRIVER_NAME " rev " DRIVER_REV);

/*	dclk_pll.fref = (PEXTDEV_Read(BOOT_0) & PEXTDEV_Def(BOOT_0_STRAP_CRYSTAL, 14318180)) ? 14318180 : 13500000; */

	NOTICE("Crystal = %uKHz", dclk_pll.fref/1000);

	card->clock = &clock;
	return EOK;
}


#include "pll.inc"                         
